% function figureHandle=Limit_Tippet_Plots_natural_LR(varargin);
%
% Plots Tippet plots from natural LR values
% Include the limits determined by the misleading evidence bounds computed
% by Royall (2000) in his paper:
%
% R. Royall, 2000. "On the probability of observing misleading statistical
% evidence." Journal of the American Statistical Association, v. 95(451),
% pp. 760-768.
% 
% Usage: tippet_plots_natural_LR_misleadingLimits({'set1',{trueHp_lrs1,trueHd_lrs1}}, {'set2',{trueHp_lrs2,trueHd_lrs2}} ,...)   
%        tippet_plots_natural_LR_misleadingLimits({'set1',{trueHp_lrs1,trueHd_lrs1}}, {'set2',{trueHp_lrs2,trueHd_lrs2}}  ,...,log10LimitToVisualize)
%        tippet_plots_natural_LR_misleadingLimits({'set1',{trueHp_lrs1,trueHd_lrs1}}, {'set2',{trueHp_lrs2,trueHd_lrs2}}  ,...,boundType)
%        tippet_plots_natural_LR_misleadingLimits({'set1',{trueHp_lrs1,trueHd_lrs1}}, {'set2',{trueHp_lrs2,trueHd_lrs2}}  ,...,boundType,log10LimitToVisualize)
%
% Input parameters: One or more entries, one entry per LR set under evaluation.
%      'set1': A string label for LR set 1
%      trueHp_lrs1: An array of the true-Hp NATURAL LRs of LR set 1
%      trueHd_lrs1: An array of the true-Hd NATURAL LRs of LR set 1
%      log10LimitToVisualize: double. log10 of the limit of the LR value to
%      visualize in the x-axis.
%      boundType: type of bound to plot in the Tippett plots. This is a
%      data structure with several mandatory fields:
%        -- Bound type name (field boundName). Different options:
%          * 'universal' (default): Universal bound according to Royall 2000.
%          * 'normalEqualVarMax': maximum Royall bound for normal
%                               distribution with equal variances.
%          * 'normalEqualVar': Royall bound for normal distribution with
%                            equal variances. It requires two additional
%                            fields:
%                            -- Difference of means (field meanDifference).
%                            -- Common variance (field commonVariance).
%                            -- Sample size (field n).
%          * 'normal': Simultaion for normal distribution with
%                            different variances. It requires five additional
%                            fields:
%                            -- Mean of Hp (field mean1).
%                            -- Mean of Hd (field mean1).
%                            -- Variance of Hp (field var1).
%                            -- Variance of Hd (field var2).
%                            -- Sample size (field n).
%
%      
function figureHandle=Limit_Tippet_Plots_natural_LR(varargin)

% number of input parameters
nInputs=length(varargin); 

if ~iscell(varargin{nInputs})
    if isstruct(varargin{nInputs}),
        boundType=varargin{nInputs};
        boundName=varargin{nInputs}.boundName;
        base_10_limit=3;
        nSystems=nInputs-1;
    elseif isnumeric(varargin{nInputs})
        if isstruct(varargin{nInputs-1}),
            boundType=varargin{nInputs-1};
            boundName=varargin{nInputs-1}.boundName;
            base_10_limit=varargin{nInputs};
            nSystems=nInputs-2;
        elseif iscell(varargin{nInputs-1}),
            boundName='universal';
            base_10_limit=varargin{nInputs};
            nSystems=nInputs-1;
        end;
    end;
else
    base_10_limit=3;
    boundName='universal';
    nSystems=nInputs;
end;

colors={'b' 'r--' 'g-.' 'c:' 'y' 'm:'};

% Too much systems are not plotted
maxTippettsTogether=length(colors);
if nSystems>maxTippettsTogether,
    disp(['ERROR: too much Tippett plots to put together (maximum ' maxTippettsTogether ').']);
    return;
end;

plo=-base_10_limit-2:0.0001:base_10_limit+2;
nPoints=length(plo);
ploNegative=plo(find(plo<0));
ploPositive=plo(find(plo>0));

% Limiting bounds
switch lower(boundName)
    
    case 'universal'
        boundNonTargets=100*(1./(10.^ploPositive));
        boundTargets=100*(1-(10.^ploNegative));
        
    case 'normalequalvar'
        if isfield(boundType,'n'),
            n=boundType.n;
        else
            error(['Field n required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'meanDifference'),
            meanDifference=boundType.meanDifference;
        else
            error(['Field meanDifference required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'commonVariance'),
            commonVariance=boundType.commonVariance;
        else
            error(['Field commonVariance required for ' lower(boundName) ' bound type']);
            return;
        end;

        commonStd=sqrt(commonVariance); 
        c=(meanDifference*sqrt(n))/commonStd;
        
        boundNonTargets=100*(normcdf(-(c/2)-(log(10.^ploPositive)./c),0,1));
        boundTargets=100*(1-(normcdf(-(c/2)-(log(10.^-ploNegative)./c),0,1)));
        
    case 'normalequalvarmax'
        boundNonTargets=100*(normcdf(-sqrt(2*log(10.^ploPositive)),0,1));
        boundTargets=100*(1-(normcdf(-sqrt(2*log(10.^(-ploNegative))),0,1)));
    
    case 'normal'
        % Checking for fields in the structure
        if isfield(boundType,'n'),
            n=boundType.n;
        else
            error(['Field n required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'mean1'),
            mean1=boundType.mean1;
        else
            error(['Field mean1 required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'mean2'),
            mean2=boundType.mean2;
        else
            error(['Field mean2 required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'var1'),
            var1=boundType.var1;
        else
            error(['Field var1 required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        if isfield(boundType,'var2'),
            var2=boundType.var2;
        else
            error(['Field var2 required for ' lower(boundName) ' bound type']);
            return;
        end;
        
        % Simulation procedure.
        % 100000 of samples of size n for each hypothesis. 
        % Step 1: generate a matrix of 100000 x n per hypothesis.
        % Step 2: compute the LR for single elements in the
        % sample.
        % Step 3: Multiply LR values n by n (equivalent to compute the "T
        % statistic").
        % Step 4: draw partial tippett plots with the resulting LR values.
        
        % Step 1:
        samplesH1=randn(1000000*n,1)*sqrt(var1) + mean1;
        samplesH2=randn(1000000*n,1)*sqrt(var2) + mean2;
        
        % Step 2:
        numeratorH1=normpdf(samplesH1,mean1,var1);
        denominatorH1=normpdf(samplesH1,mean2,var2);
        lrsH1=numeratorH1./denominatorH1;
        
        numeratorH2=normpdf(samplesH2,mean1,var1);
        denominatorH2=normpdf(samplesH2,mean2,var2);
        lrsH2=numeratorH2./denominatorH2;
        
        if isinf(lrsH1),
            error('ERROR: infinite values for H1 LR values. Probably the distirbutions used are too inapropriate');
            return;
        elseif isnan(lrsH1),
            error('ERROR: nan values for H1 LR values. Probably the distirbutions used are too inapropriate');
            return;
        end;
        
        if isinf(lrsH2),
            error('ERROR: infinite values for H2 LR values. Probably the distirbutions used are too inapropriate');
            return;
        elseif isnan(lrsH2),
            error('ERROR: nan values for H2 LR values. Probably the distirbutions used are too inapropriate');
            return;
        end;
        
        % Step 3:
        lrsH1=reshape(lrsH1,n,1000000);
        if n>1,
            lrsH1=prod(lrsH1);
        end;
        
        lrsH2=reshape(lrsH2,n,1000000);
        if n>1,
            lrsH2=prod(lrsH2);
        end;
        
        
        % Step 4 limits on Hd:
        % Conversion to base-10 log-LRs
        nonTargets=log10(lrsH2);
        count=histc(nonTargets,ploPositive);

        % This is the rest of the count which comes out from the scale of the
        % plot
        restoCount=length(find(nonTargets<(ploPositive(1))));

        boundNonTargets=(1-((restoCount+cumsum(count))/length(nonTargets)))*100;
        
        % Step 4 limits on Hp:
        % Conversion to base-10 log-LRs
        targets=log10(lrsH1);
        count=histc(targets,ploNegative);
        % count=histc(nonTargets,[min(ploNegative)-2 min(ploNegative)-1 ploNegative max(ploNegative)+1 max(ploNegative)+2]);
        % count=count([3:length(count)-2]);

        % This is the rest of the count which comes out from the scale of the
        % plot
        restoCount=length(find(targets<(ploNegative(1))));

        boundTargets=(1-((restoCount+cumsum(count))/length(targets)))*100;
        
    otherwise
        error(['Unknown method: ' boundType]);
        return;
end;


figureHandle=figure;
hold on;
a=get(figureHandle,'currentAxes');
set(a,'FontSize',16);
set(gcf,'Position',get(0,'ScreenSize')./1.2);
legendsToPlot=cell(1,nSystems+2);



% PLOTTING BOUNDS
% For targets
plot(ploNegative,boundTargets,'k:','LineWidth',3);
%legendsToPlot(1)={['Bound H_p (' boundName ')']};
legendsToPlot(1)={['H_p Bound']};
% For non-targets
plot(ploPositive,boundNonTargets,'k-.','LineWidth',3);
%legendsToPlot(2)={['Bound H_d (' boundName ')']};
legendsToPlot(2)={['H_d Bound']};

% Drawing Tippett plots. All systems in the same figure
% Only if LR vectors are not empty
data=varargin{1};
lrs=data{2};
lrsT=lrs{1};
lrsNT=lrs{2};
if ((length(lrsT)>1) && (length(lrsNT)>1)),

    % Target lines
    for i=1:nSystems
        data = varargin{i}; 
        name = data{1};
        lrs = data{2};

        % Conversion to base-10 log-LRs
        targets=log10(lrs{1});
        count=histc(targets,plo);

        % This is the rest of the count which comes out from the scale of the
        % plot
        restoCount=length(find(targets<(plo(1))));

        tippettTarget=(1-((restoCount+cumsum(count))/length(targets)))*100;

        plot(plo,tippettTarget,colors{i},'LineWidth',2);
        legendsToPlot(i+2)={name};
    end;

    legend(legendsToPlot,'Location','NorthEast');


    for i=1:nSystems
        data = varargin{i}; 
        name = data{1};
        lrs = data{2};

        % Conversion to base-10 log-LRs
        nonTargets=log10(lrs{2});
        count=histc(nonTargets,[min(plo)-1 plo max(plo)+1]);
        count=count([2:length(count)-1]);

        % This is the rest of the count which comes out from the scale of the
        % plot
        restoCount=length(find(nonTargets<(plo(1))));

        tippettNonTarget=(1-((restoCount+cumsum(count))/length(nonTargets)))*100;

        plot(plo,tippettNonTarget,colors{i},'LineWidth',2);
    end;
else,
    legend(legendsToPlot(1:2),'Location','NorthEast');
end;

xlabel('Log_{10}(LR) Greater Than','FontSize',14);
ylabel('Proportion of cases (%)','FontSize',14);
axis([-base_10_limit base_10_limit 0 100]);
grid;
